@@ -1,19 +1,42 @@
 import pandas as pd
 from .utils import generate_groups_data_flat, generate_groups_data_matrix
 import calendar
+from urllib import request
 from pathlib import Path
+import os
 
 
 class PreprocessDatasets:
 
     def __init__(self, dataset):
         self.dataset = dataset
+        self.api = 'http://www.machinelearningtimeseries.com/apidownload/'
+        self._create_directories()
 
     @staticmethod
-    def _prison():
-        # prison dataset
-        path = Path(__file__).parent / './original_datasets/prisonLF.csv'
-        prison = pd.read_csv(path, sep=",")
+    def _create_directories():
+        # Create directory to store original datasets if does not exist
+        Path("./original_datasets").mkdir(parents=True, exist_ok=True)
+        # Create directory to store transformed datasets if does not exist
+        Path("./transformed_datasets").mkdir(parents=True, exist_ok=True)
+
+    def _get_dataset(self):
+        path = f'./original_datasets/{self.dataset}.csv'
+        # Download the original file if it does not exist
+        if not os.path.isfile(path):
+            try:
+                request.urlretrieve(f'{self.api}{self.dataset}', path)
+                return pd.read_csv(path, sep=",")
+            except:
+                print('It is not possible to download the dataset at this time!')
+                return pd.DataFrame()
+        else:
+            return pd.read_csv(path, sep=",")
+
+    def _prison(self):
+        prison = self._get_dataset()
+        if prison.empty:
+            return {}
         prison = prison.drop('Unnamed: 0', axis =1)
         prison['t'] = prison['t'].astype('datetime64[ns]')
         prison_pivot = prison.pivot(index='t',columns=['state', 'gender', 'legal'], values='count')
@@ -31,11 +54,10 @@ class PreprocessDatasets:
         groups = generate_groups_data_matrix(groups)
         return groups
 
-    @staticmethod
-    def _tourism():
-        # tourism dataset
-        path = Path(__file__).parent / './original_datasets/TourismData_v3.csv'
-        data = pd.read_csv(path)
+    def _tourism(self):
+        data = self._get_dataset()
+        if data.empty:
+            return {}
         data['Year'] = data['Year'].fillna(method='ffill')
 
         d = dict((v,k) for k,v in enumerate(calendar.month_name))
@@ -55,5 +77,5 @@ class PreprocessDatasets:
         return groups
 
     def apply_preprocess(self):
-        dataset_new = getattr(PreprocessDatasets, '_' + self.dataset)()
+        dataset_new = getattr(PreprocessDatasets, '_' + self.dataset)(self)
         return dataset_new
